var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Connector } from '@web3-react/types';
function parseChainId(chainId) {
    return typeof chainId === 'string' ? Number.parseInt(chainId) : chainId;
}
export class WalletConnect extends Connector {
    /**
     * @param options - Options to pass to `@walletconnect/ethereum-provider`
     * @param connectEagerly - A flag indicating whether connection should be initiated when the class is constructed.
     */
    constructor(actions, options, connectEagerly = true, treatModalCloseAsError = true) {
        super(actions);
        this.disconnectListener = (error) => {
            this.actions.reportError(error);
        };
        this.chainChangedListener = (chainId) => {
            this.actions.update({ chainId: parseChainId(chainId) });
        };
        this.accountsChangedListener = (accounts) => {
            this.actions.update({ accounts });
        };
        this.options = options;
        this.treatModalCloseAsError = treatModalCloseAsError;
        if (connectEagerly) {
            this.eagerConnection = this.initialize(true);
        }
    }
    initialize(connectEagerly, chainId) {
        return __awaiter(this, void 0, void 0, function* () {
            let cancelActivation;
            if (connectEagerly) {
                cancelActivation = this.actions.startActivation();
            }
            return import('@walletconnect/ethereum-provider').then((m) => {
                this.provider = new m.default(Object.assign(Object.assign({}, this.options), (chainId ? { chainId } : undefined)));
                this.provider.on('disconnect', this.disconnectListener);
                this.provider.on('chainChanged', this.chainChangedListener);
                this.provider.on('accountsChanged', this.accountsChangedListener);
                if (connectEagerly) {
                    if (this.provider.connected) {
                        return Promise.all([
                            this.provider.request({ method: 'eth_chainId' }),
                            this.provider.request({ method: 'eth_accounts' }),
                        ])
                            .then(([chainId, accounts]) => {
                            if (accounts === null || accounts === void 0 ? void 0 : accounts.length) {
                                this.actions.update({ chainId: parseChainId(chainId), accounts });
                            }
                            else {
                                throw new Error('No accounts returned');
                            }
                        })
                            .catch((error) => {
                            console.debug('Could not connect eagerly', error);
                            cancelActivation();
                        });
                    }
                    else {
                        cancelActivation();
                    }
                }
            });
        });
    }
    /**
     * Initiates a connection.
     *
     * @param desiredChainId - If defined, indicates the desired chain to connect to. If the user is
     * already connected to this chain, no additional steps will be taken. Otherwise, the user will be prompted to switch
     * to the chain, if their wallet supports it.
     */
    activate(desiredChainId) {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            // this early return clause catches some common cases if we're already connected
            if ((_a = this.provider) === null || _a === void 0 ? void 0 : _a.connected) {
                if (!desiredChainId)
                    return;
                if (desiredChainId === this.provider.chainId)
                    return;
                const desiredChainIdHex = `0x${desiredChainId.toString(16)}`;
                return this.provider
                    .request({
                    method: 'wallet_switchEthereumChain',
                    params: [{ chainId: desiredChainIdHex }],
                })
                    .catch(() => {
                    void 0;
                });
            }
            // if we're trying to connect to a specific chain, we may have to re-initialize
            if (desiredChainId && desiredChainId !== ((_b = this.provider) === null || _b === void 0 ? void 0 : _b.chainId)) {
                yield this.deactivate();
            }
            this.actions.startActivation();
            if (!this.eagerConnection) {
                this.eagerConnection = this.initialize(false, desiredChainId);
            }
            yield this.eagerConnection;
            const wasConnected = !!((_c = this.provider) === null || _c === void 0 ? void 0 : _c.connected);
            try {
                // these are sequential instead of parallel because otherwise, chainId defaults to 1 even
                // if the connecting wallet isn't on mainnet
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                const accounts = yield this.provider.request({ method: 'eth_requestAccounts' });
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                const chainId = parseChainId(yield this.provider.request({ method: 'eth_chainId' }));
                if (!desiredChainId || desiredChainId === chainId) {
                    return this.actions.update({ chainId, accounts });
                }
                // because e.g. metamask doesn't support wallet_switchEthereumChain, we have to report first-time connections,
                // even if the chainId isn't necessarily the desired one. this is ok because in e.g. rainbow,
                // we won't report a connection to the wrong chain while the switch is pending because of the re-initialization
                // logic above, which ensures first-time connections are to the correct chain in the first place
                if (!wasConnected)
                    this.actions.update({ chainId, accounts });
                // try to switch to the desired chain, ignoring errors
                if (desiredChainId && desiredChainId !== chainId) {
                    const desiredChainIdHex = `0x${desiredChainId.toString(16)}`;
                    return (_d = this.provider) === null || _d === void 0 ? void 0 : _d.request({
                        method: 'wallet_switchEthereumChain',
                        params: [{ chainId: desiredChainIdHex }],
                    }).catch(() => {
                        void 0;
                    });
                }
            }
            catch (error) {
                // this condition is a bit of a hack :/
                // if a user triggers the walletconnect modal, closes it, and then tries to connect again, the modal will not trigger.
                // the logic below prevents this from happening
                if (error.message === 'User closed modal') {
                    yield this.deactivate(this.treatModalCloseAsError ? error : undefined);
                }
                else {
                    this.actions.reportError(error);
                }
            }
        });
    }
    /** {@inheritdoc Connector.deactivate} */
    deactivate(error) {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            (_a = this.provider) === null || _a === void 0 ? void 0 : _a.off('disconnect', this.disconnectListener);
            (_b = this.provider) === null || _b === void 0 ? void 0 : _b.off('chainChanged', this.chainChangedListener);
            (_c = this.provider) === null || _c === void 0 ? void 0 : _c.off('accountsChanged', this.accountsChangedListener);
            yield ((_d = this.provider) === null || _d === void 0 ? void 0 : _d.disconnect());
            this.provider = undefined;
            this.eagerConnection = undefined;
            this.actions.reportError(error);
        });
    }
}
